package models

import (
	"crypto/sha256"
	"encoding/base64"
	"encoding/hex"

	"github.com/jackc/pgtype"
)

// TemplateDB is a Template stored in db
// dbtable:"template" dbpkey:"id"
type TemplateDB struct {
	Template

	ID uint64 `db:"id" json:"-"`

	InternalVersion uint64       `db:"internal_version" json:"-"`
	ETag            string       `db:"etag" json:"-"`
	BytesBody       pgtype.Bytea `db:"body"             json:"-"`
}

func (t *TemplateDB) CalcETag() string {
	hash := sha256.New()

	_, _ = hash.Write([]byte(t.Account))
	_, _ = hash.Write([]byte(t.Name))
	_, _ = hash.Write([]byte(t.Version))
	_, _ = hash.Write([]byte(t.Title))
	_, _ = hash.Write([]byte(t.Description))
	_, _ = hash.Write([]byte(t.Produces))
	_, _ = hash.Write([]byte(t.CreatedAt.String()))
	_, _ = hash.Write([]byte(t.Language))
	_, _ = hash.Write([]byte(t.Locale))
	_ = t.Metadata.writeTo(hash)
	_ = t.StaticMetadata.writeTo(hash)
	_, _ = hash.Write(t.BytesBody.Bytes)

	return hex.EncodeToString(hash.Sum(nil))
}

// FromDB loads the db fields into the api facing fields.
func (t *TemplateDB) FromDB() {
	t.BodyFormat = BodyFormatText
	t.Body = string(t.BytesBody.Bytes)
}

// ToDB copies the api facing files into the db fields.
func (t *TemplateDB) ToDB() error {
	b, err := t.Bytes()
	if err != nil {
		return err
	}

	if err := t.BytesBody.Set(b); err != nil {
		return err
	}

	t.ETag = t.CalcETag()

	return nil
}

// ForceBase64 makes sure the body is base64 encoded.
func (t *TemplateDB) ForceBase64() {
	if t.BodyFormat == BodyFormatText {
		t.Body = base64.StdEncoding.EncodeToString([]byte(t.Body))
		t.BodyFormat = BodyFormatB64
	}
}
